import os
import zipfile
from functools import reduce
import io
from typing import Callable, Iterable, Optional, Union


FileType = Union[str, os.PathLike, io.BytesIO]


def default_filterfunc(name: str) -> bool:
    if os.path.basename(name) == '.DS_Store':
        return False
    if name.startswith('__MACOSX'):
        return False
    return True


def merge_zip(
    file1: FileType,
    file2: FileType,
    filterfunc: Callable = default_filterfunc,
) -> io.BytesIO:
    """
    Merge `zip1` and `zip2`.

    The second zip will overwrite the first zip.
    """
    zip1 = zipfile.ZipFile(file1)
    zip2 = zipfile.ZipFile(file2)
    zip_buffer = io.BytesIO()
    with zipfile.ZipFile(zip_buffer, 'a') as zip_file:
        written = []

        for filename in filter(filterfunc, zip2.namelist()):
            zip_file.writestr(filename, zip2.open(filename).read())
            written.append(filename)

        for filename in filter(filterfunc, zip1.namelist()):
            if filename not in written:
                zip_file.writestr(filename, zip1.open(filename).read())

    zip_buffer.seek(0)
    return zip_buffer


def merge_many_zip(files: Iterable[FileType]) -> io.BytesIO:
    return reduce(merge_zip, files)


def force_zip(file: FileType, name: Optional[str] = None) -> Union[io.BytesIO, FileType]:
    if zipfile.is_zipfile(file):
        # Method `is_zipfile` change the stream position to the last byte.
        file.seek(0)
        return file
    else:
        assert name is not None, 'If `file` is not zipfile, argument `name` is required.'
        zip_buffer = io.BytesIO()

        if isinstance(file, os.PathLike):
            file = os.fspath(file)
        if isinstance(file, str):
            file = io.open(file, 'rb')

        with zipfile.ZipFile(zip_buffer, 'a') as zip_file:
            file.seek(0)
            zip_file.writestr(os.path.basename(name), file.read())
        return zip_buffer



import asyncio
import functools

import sentry_sdk


def sentry_child_transaction(name: str):
    def sentry_context(name: str):
        span = sentry_sdk.Hub.current.scope.span
        if span is None:
            return sentry_sdk.start_transaction(name=name)
        else:
            return span.start_child(op=name)

    def wrapper(func):
        if not asyncio.iscoroutinefunction(func):

            @functools.wraps(func)
            def wrapped(*args, **kwargs):
                with sentry_context(name=name):
                    return func(*args, **kwargs)

        else:

            @functools.wraps(func)
            async def wrapped(*args, **kwargs):
                with sentry_context(name=name):
                    return await func(*args, **kwargs)

        return wrapped

    return wrapper



from django.http import Http404
from django.core.exceptions import PermissionDenied

from rest_framework import exceptions
from rest_framework.views import set_rollback
from rest_framework.response import Response


def api_exception_handler(exc, context):

    if isinstance(exc, Http404):
        exc = exceptions.NotFound()
    elif isinstance(exc, PermissionDenied):
        exc = exceptions.PermissionDenied()

    if isinstance(exc, exceptions.APIException):
        headers = {}
        if getattr(exc, 'auth_header', None):
            headers['WWW-Authenticate'] = exc.auth_header
        if getattr(exc, 'wait', None):
            headers['Retry-After'] = '%d' % exc.wait

        error = {}
        error['reason'] = exc.__class__.__name__
        error['status'] = exc.status_code
        error['code'] = exc.default_code
        error['message'] = exc.default_detail
        error['detail'] = exc.get_full_details()

        set_rollback()
        return Response(
            {'error': error},
            status=exc.status_code,
            headers=headers,
        )

    return None



def postgres_set(connection, parameter, value):
    with connection.cursor() as cursor:
        cursor.execute('SET "{}" = %s'.format(parameter), (value,))


def set_word_similarity_threshold(value):
    from django.db import connection

    postgres_set(connection, 'pg_trgm.word_similarity_threshold', value)



#!/usr/bin/env python3
import argparse
import os

from fan_tools.unix import succ

parser = argparse.ArgumentParser(description='ci script: build, push branches/tags, cache')
parser.add_argument('-r', '--repo', dest='repo', help='push repo')
parser.add_argument('-p', '--project', dest='project')

parser.add_argument('-t', '--tag', dest='tag', default=os.environ.get('CI_COMMIT_TAG'))
parser.add_argument('--branch-name', dest='branch', default=os.environ.get('CI_COMMIT_REF_NAME'))
parser.add_argument('--push-branches', dest='push_branches', default='master,staging')

parser.add_argument('--docker-file', dest='docker_file', default='Dockerfile')
parser.add_argument('--test', help='test command', default='true')
parser.add_argument(
    '--cache-name',
    help='image cache name',
    default='{}_cache'.format(os.environ.get('CI_PROJECT_NAME', 'default')),
)
parser.add_argument(
    '--temp-name', dest='temp_name', help='temp image name', default=os.environ.get('CI_JOB_ID')
)
args = parser.parse_args()

FULL_BRANCH = '{}/{}:{}'.format(args.repo, args.project, args.branch)
FULL_TAG = '{}/{}:{}'.format(args.repo, args.project, args.tag)


def build():
    sha = args.temp_name
    cmd = ['docker build -t', sha, '-f', args.docker_file, '.']
    print('Build: {}'.format(' '.join(cmd)))
    succ(cmd)


def run_test():
    succ([args.test], check_stderr=False)


def push_branch():
    if args.branch not in args.push_branches.split(','):
        print('Skip push branch: {}'.format(args.branch))
        return
    print('Push branch {}'.format(args.branch))
    succ(['docker tag ', args.temp_name, FULL_BRANCH])
    succ(['docker push ', FULL_BRANCH])


def push_tag():
    if not args.tag:
        print('Skip push tag')
        return
    print('Push tag: {}'.format(args.tag))
    succ(['docker tag', args.temp_name, FULL_TAG])
    succ(['docker push', FULL_TAG])


def cache():
    print('Cache image as: {}'.format(args.cache_name))
    succ(['docker tag', args.temp_name, args.cache_name, '&& docker rmi', args.temp_name])


def main():
    build()
    run_test()
    push_branch()
    push_tag()
    cache()


if __name__ == '__main__':
    main()



from docutils import nodes
from docutils.statemachine import StringList


class ParseMixin:
    """
    Providing ability to generate nodes from plain text rst-strings.
    It's easier sometimes to generate rst from templates without manual
    work with nodes.
    """

    def rst_lines(self, lines):
        self.state_machine.insert_input(lines, '<src>')

    def parse_lines(self, lines):
        node = nodes.Element('')
        line = StringList(lines, '<src>')
        self.state.nested_parse(line, self.content_offset, node)
        return node.children

    def parse(self, line):
        return self.parse_lines([line])[0]



from django.conf import settings
from django.core.management import BaseCommand
from django.utils import autoreload


class BaseAutoReloadCommand(BaseCommand):
    """
    Django-command base class with auto-reload.

    It is recommended to use it when implementing long-running workers ONLY IN DEVELOPMENT.
    The command uses the auto-reload mechanism from django and will reload the process if
    django-reloader detects changes in the code.

    The class retains the functionality of BaseCommand,
    only replaces `handle` method with 'run_command'.
    """

    def handle(self, *args, **kwargs):
        if settings.DEBUG:
            autoreload.run_with_reloader(self.run_command, *args, **kwargs)
        else:
            self.run_command(*args, **kwargs)

    def run_command(self, *args, **kwargs):
        raise NotImplementedError('Method `run_command not implemented.`')



from django.conf import settings
from django.core.mail.message import EmailMultiAlternatives
from django.template.loader import render_to_string
from django.utils.module_loading import import_string


class Mail:
    template_subject_suffix = '_subject.txt'
    template_message_suffix = '_body.txt'
    template_message_html_suffix = '_body.html'

    def __init__(self, *args, **kwargs):

        self.sender = kwargs.get(
            'sender',
            getattr(
                settings,
                'DEFAULT_FROM_EMAIL',
                None,
            ),
        )
        self.recipient_list = kwargs.get('recipient_list', [])

        self.template_name = kwargs.get('template_name')
        self.context = kwargs.get('context', {})

        self.fail_silently = kwargs.get('fail_silently', False)

    def send(self):

        mail = EmailMultiAlternatives(
            render_to_string(
                self.template_name + Mail.template_subject_suffix,
                self.context,
            ).strip(),
            render_to_string(
                self.template_name + Mail.template_message_suffix,
                self.context,
            ),
            self.sender,
            self.recipient_list,
            connection=import_string(settings.EMAIL_BACKEND)(
                fail_silently=self.fail_silently,
            ),
        )
        mail.attach_alternative(
            render_to_string(
                self.template_name + Mail.template_message_html_suffix,
                self.context,
            ),
            'text/html',
        )
        return mail.send()



class ApiUrls:
    def __init__(self, base, urls):
        self.base = base
        self.urls = urls

    def __getattr__(self, name):
        url_tpl = self.urls.get(name)
        if url_tpl is None:
            raise AttributeError
        return '{}{}'.format(self.base, url_tpl).format



import logging
import os
import socket
from contextlib import suppress
from pathlib import Path

from fan_tools import const
from fan_tools.fan_logging import setup_logger


default_log_dir = os.environ.get('LOG_DIR', '.logs')
log_file_name = os.environ.get('LOG_FILE_NAME', 'fan_logging')


def enable_otel_logger(
    log_dir=default_log_dir,
    log_file=log_file_name,
    vector_info=None,
    env_vars=const.ENV_VARS,
    log_levels=const.LOG_LEVELS,
):
    log_dir = Path(default_log_dir)
    log_dir.mkdir(parents=True, exist_ok=True)
    handlers = {}
    if vector_info is None:
        vector_info = const.OTEL['VECTOR']

    with suppress(socket.gaierror):
        socket.gethostbyname(vector_info[0])
        handlers = {
            'vector': {
                'level': 'DEBUG',
                'class': 'fan_tools.fan_logging.handlers.SysLogHandler',
                'address': vector_info,
                'formatter': 'json',
            }
        }

    params = {
        'root_dir': log_dir,
        'enable_stdout': True,
        'stdout_level': 'INFO',
        'json_formatter': 'fan_tools.fan_logging.JSFormatter',
        'handlers': handlers,
    }

    params['json_formatter'] = 'fan_tools.otel.OtelJSFormatter'
    params['enable_stdout'] = False
    params['stdout_level'] = 'CRITICAL'
    params['json_params'] = {
        'env_vars': env_vars,
    }

    with suppress(ValueError):
        setup_logger(log_file, **params)

    for level, names in log_levels.items():
        for name in names:
            logging.getLogger(name).setLevel(level)



import asyncio
import functools
import itertools
import logging
import os
import sys
import warnings
import zipfile
from decimal import Decimal, ROUND_HALF_UP
from pathlib import Path
from typing import Any, BinaryIO, Dict, Iterable, List, Optional, Sequence, Union


default_logger = logging.getLogger(__name__)


def execfile(fname: Union[str, Path], _globals: Dict[str, Any], _locals: Dict[str, Any]):
    """
    Usage: execfile('path/to/file.py', globals(), locals())
    """
    if os.path.exists(fname):
        with open(fname) as f:
            code = compile(f.read(), os.path.basename(fname), 'exec')
            exec(code, _globals, _locals)
            return True
    else:
        return False


def rel_path(path: str, check: bool = False, depth: int = 1) -> Path:
    d = Path(sys._getframe(depth).f_code.co_filename).parent
    full = (d / path).resolve()
    if check and not full.exists():
        raise Exception('No such path: {!r}'.format(full))
    return full


def py_rel_path(*args, **kwargs):
    message = 'Use `rel_path` function instead'
    warnings.warn(message, DeprecationWarning, stacklevel=2)
    return rel_path(*args, depth=2, **kwargs)


def auto_directory(path: Union[str, Path]) -> Path:
    """
    if you're using py.path you make do that as:
    py.path.local(full_path).ensure_dir()
    """
    if isinstance(path, str):
        dir_path = rel_path(path, depth=2)
    else:
        dir_path = path
    dir_path.mkdir(exist_ok=True, parents=True)
    return dir_path


def usd_round(amount: Union[str, Decimal]) -> Decimal:
    if isinstance(amount, str):
        amount = Decimal(amount)
    return amount.quantize(Decimal('.01'), rounding=ROUND_HALF_UP)


def root_directory(target: Union[Path, BinaryIO], exclude: Optional[List[str]] = None) -> Path:
    """Returns the folder that contains the files."""
    exclude = exclude or []
    if isinstance(target, Path):
        files = [f for f in target.glob('**/*') if f.is_file()]
    elif hasattr(target, 'read'):
        with zipfile.ZipFile(target, 'r') as zip_ref:
            files = [
                Path(zip_item.filename) for zip_item in zip_ref.infolist() if not zip_item.is_dir()
            ]
    else:
        raise Exception('Unsupported target object')
    paths = [f for f in files if not any(filter(lambda i: i in str(f), exclude))]
    if len(paths) > 1:
        prefix = os.path.commonpath(paths)
        prefix = '' if prefix == '/' else prefix
        root = Path(prefix)
    elif paths:
        root = paths[0].parent
    else:
        root = Path('')
    return root


def dict_contains(superset: Dict[Any, Any], subset: Dict[Any, Any]) -> bool:
    """
    partial comparison dictionaries

    $ dict_contains({'a': 1, 'b': 2}, {'a': 1}) # => True
    $ dict_contains({'a': 1, 'b': 2}, {'b': 1}) # => True
    $ dict_contains({'a': 1, 'b': 2}, {'c': 3}) # => False
    $ dict_contains({'a': 1, 'b': 2}, {'b': 2}) # => False
    """

    for k, v in subset.items():
        if k not in superset:
            return False
        if isinstance(v, dict):
            if not dict_contains(superset[k], subset[k]):
                return False
        else:
            if v != superset[k]:
                return False
    return True


def slide(iterable: Iterable[Any], size=2) -> Iterable[Any]:
    return itertools.zip_longest(*[itertools.islice(iterable, i, sys.maxsize) for i in range(size)])


def dot_get(path: str, dct: Dict[str, Any], default: Any = [], sep: str = '.'):
    """
    get nested data from dictionary

    $ dot_get('a.b.c', {'a': {'b': {'c': [1]}}}) # => [1]
    """
    for key in path.split(sep):
        if key not in dct or not isinstance(dct, dict):
            return default
        dct = dct[key]
    return dct


def expand_dot(pattern: Dict[str, Any]) -> Dict[str, Any]:
    """
    {'a.b.c': 1} => {'a':{'b':{'c': 1}}}
    """
    out = {}
    for k, v in pattern.items():
        if '.' in k:
            curr = out
            for sub_k, has_next in slide(k.split('.')):
                if has_next:
                    curr = curr.setdefault(sub_k, {})
                else:
                    curr[sub_k] = v
        else:
            out[k] = v
    return out


_initial_missing = object()


async def areduce(function, sequence, initial=_initial_missing):
    """
    asynchronous implementation of the reduce function
    based on: `functools.reduce`

    Example:
    async def asum_two(a, b):
        return a + b

    await areduce(asum_two, [1, 2, 3, 4, 5])
    """
    it = iter(sequence)

    if initial is _initial_missing:
        try:
            value = next(it)
        except StopIteration:
            raise TypeError('areduce() of empty sequence with no initial value') from None
    else:
        value = initial

    for element in it:
        value = await function(value, element)

    return value


def chunks(lst: Sequence, n: int):
    """Yield successive n-sized chunks from lst."""
    for i in range(0, len(lst), n):
        yield lst[i : i + n]


def retry(exceptions=Exception, tries=-1, logger=default_logger):
    """
    Executes a function and retries it if it failed.

    :param exceptions: an exception or a tuple of exceptions to catch. default: Exception.
    :param tries: the maximum number of attempts. default: -1 (infinite).
    :param logger: will record a call retry warning.
    """

    def decorator(func):
        if asyncio.iscoroutinefunction(func):

            @functools.wraps(func)
            async def wrapper(*args, **kwargs):
                nonlocal tries
                f = functools.partial(func, *args, **kwargs)
                while tries:
                    try:
                        return await f()
                    except exceptions as e:
                        tries -= 1
                        if not tries:
                            raise
                        logger.exception('%s, retrying...', e)

        else:

            @functools.wraps(func)
            def wrapper(*args, **kwargs):
                nonlocal tries
                f = functools.partial(func, *args, **kwargs)
                while tries:
                    try:
                        return f()
                    except exceptions as e:
                        tries -= 1
                        if not tries:
                            raise
                        logger.exception('%s, retrying...', e)

        return wrapper

    return decorator



import os
import uuid


class UploadNameGenerator(object):

    def __init__(self, model_name, field_name):
        self.model_name = model_name
        self.field_name = field_name

    def deconstruct(self):
        return (
            'fan_tools.django.UploadNameGenerator',
            (),
            {
                'model_name': self.model_name,
                'field_name': self.field_name,
            },
        )

    def __call__(self, instance, filename):
        return os.path.join(
            'static',
            self.model_name,
            '%s-%s-%s%s' % (
                self.model_name,
                self.field_name,
                uuid.uuid1(),
                os.path.splitext(filename)[1],
            ),
        )



import logging
from contextlib import contextmanager

from django.contrib.auth import get_user_model
from django.test import TestCase
from django.contrib.auth.models import Permission, Group
from django.contrib.contenttypes.models import ContentType
from rest_framework.test import APIClient

from fan_tools.testing.meta import PropsMeta


def get_ids(data, path='id'):
    if isinstance(data, list):
        for item in data:
            yield from get_ids(item, path)
    elif isinstance(data, dict):
        part, *rest = path.split('.', 1)
        if rest:
            if part in data:
                yield from get_ids(data.get(part), rest[0])
        else:
            yield data.get(part)
    else:
        yield None


class BaseTest(TestCase, metaclass=PropsMeta):
    log = logging.getLogger('BaseTest')
    content_format = 'json'
    default_client = 'c'

    _overridden_settings = {
        'DEFAULT_FILE_STORAGE': 'django.core.files.storage.FileSystemStorage',
        'BROKER_BACKEND': 'memory',
        'CACHES': {
            'default': {
                'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
                'TIMEOUT': 60 * 15
            }
        }
    }

    @classmethod
    def setUpTestData(cls):
        # Cache all methods decorated with class_prop
        # before running any test from test case
        for name in dir(cls):
            getattr(cls, name)

    def is_empty(self, name):
        return not hasattr(self, name)

    @classmethod
    def create_user(cls, username, groups=(), permissions=()):
        pwd = username
        User = get_user_model()
        exists = User.objects.filter(username=username).first()
        if exists:
            user = exists
        else:
            user = User.objects.create_user(username, '%s@fan.com' % username, pwd)
            for name in groups:
                group = Group.objects.filter(name=name).first()
                if not group:
                    group = Group.objects.create(name=name)
                user.groups.add(group)

            for model, codename in permissions:
                content_type = ContentType.objects.get_for_model(model)
                try:
                    p = Permission.objects.get(codename=codename, content_type=content_type)
                except Permission.DoesNotExist:
                    p = Permission.objects.create(codename=codename, content_type=content_type)
                user.user_permissions.add(p)
        client = APIClient()
        client.login(username=username, password=pwd)

        # refresh permissions
        user.refresh_from_db()
        user.client = client
        return user

    def prop__u(self):
        return self.create_user('simple_user')

    def prop__c(self):
        ''' Authorized client '''
        return self.create_user('authorized', [], [])

    def prop__c2(self):
        ''' Unauthorized client '''
        return APIClient()

    def prop__superuser(self):
        User = get_user_model()
        exists = User.objects.filter(username='su').first()
        if exists:
            return exists
        return User.objects.create_superuser('su', 'su@u.com', 'su')

    def prop__su(self):
        self.superuser
        su = APIClient()
        su.login(username='su', password='su')
        self.superuser.client = su
        return self.superuser

    def _method(self, method, url, data, client):
        User = get_user_model()
        if not client:
            if isinstance(self.default_client, User):
                client = self.default_client
            else:
                client = getattr(self, self.default_client)
        if isinstance(client, User):
            client = client.client
        return getattr(client, method)(url, data, format=self.content_format)

    def post(self, url, data, client=None):
        return self._method('post', url, data, client)

    def patch(self, url, data, client=None):
        return self._method('patch', url, data, client)

    def put(self, url, data, client=None):
        return self._method('put', url, data, client)

    def get(self, url, data={}, client=None):
        return self._method('get', url, data, client)

    def delete(self, url, data={}, client=None):
        return self._method('delete', url, data, client)

    def json_method(self, method, *args, **kwargs):
        cf = self.content_format
        self.content_format = 'json'
        r = getattr(self, method)(*args, **kwargs)
        self.content_format = cf
        return r

    def get_json(self, *args, **kwargs):
        r = self.json_method('get', *args, **kwargs)
        assert r.status_code == 200, r.status_code
        return r.json()

    def get_results(self, *args, **kwargs):
        return self.get_json(*args, **kwargs)['results']

    def post_json(self, *args, **kwargs):
        r = self.json_method('post', *args, **kwargs)
        assert r.status_code == 201, (r.status_code, r.content)
        return r.json()

    def patch_json(self, *args, **kwargs):
        r = self.json_method('patch', *args, **kwargs)
        assert r.status_code == 200, (r.status_code, r.content)
        return r.json()

    def put_json(self, *args, **kwargs):
        r = self.json_method('put', *args, **kwargs)
        assert r.status_code == 200, r.status_code
        return r.json()

    def delete_json(self, *args, **kwargs):
        r = self.json_method('delete', *args, **kwargs)
        assert r.status_code == 204, r.status_code
        assert not r.content, r.content

    @contextmanager
    def set_client(self, client):
        """
        temporary change default client for REST queries
        you can use string name or User instance as a client
        """
        old_client = self.default_client
        try:
            self.default_client = client
            yield
        finally:
            self.default_client = old_client

    def assert_items(self, expected, data, path='id'):
        """
        Extract item from `data` by `path` and compare with `expected` items
        :param expected: List of expected items.
        :param data: Tested data
        :param path: Path to the item
        Usage:
        >>> self.assert_items((1, 3, 5, 8), [
        ...     {
        ...         'name': 'bob',
        ...         'wines': [
        ...             {'name': 'wine1', 'attributes': [
        ...                 {'color': 'red', 'id': 1},
        ...                 {'color': 'green', 'id': 5},
        ...             ]},
        ...             {'name': 'wine2', 'attributes': [
        ...                 {'color': 'red', 'id': 3},
        ...             ]},
        ...         ]
        ...     },
        ...     {
        ...         'name': 'bob',
        ...         'wines': [
        ...             {'name': 'wine3', 'attributes': [
        ...                 {'color': 'red', 'id': 8},
        ...             ]},
        ...         ]
        ...     }
        ... ], 'wines.attributes.id')
        """
        actual = list(get_ids(data, path))
        assert len(expected) == len(actual), ('Expected and actual data length is not equal:\n'
                                              'len({!r}) != len({!r})'.format(expected, actual))
        assert set(expected) == set(actual), ('Expected ids are not equal to actual ids:\n'
                                              '{!r} != {!r}'.format(set(expected), set(actual)))



import logging

import io
from PIL import Image


# From: https://github.com/matthewwithanm/pilkit/blob/master/pilkit/processors/base.py#L113-L170
from django.core.files.uploadedfile import InMemoryUploadedFile


class TransposeError(Exception): pass


class Transpose:
    """
    Rotates or flips the image.
    """

    log = logging.getLogger('Transpose')

    AUTO = 'auto'
    FLIP_HORIZONTAL = Image.FLIP_LEFT_RIGHT
    FLIP_VERTICAL = Image.FLIP_TOP_BOTTOM
    ROTATE_90 = Image.ROTATE_90
    ROTATE_180 = Image.ROTATE_180
    ROTATE_270 = Image.ROTATE_270

    methods = [AUTO]
    _EXIF_ORIENTATION_STEPS = {
        1: [],
        2: [FLIP_HORIZONTAL],
        3: [ROTATE_180],
        4: [FLIP_VERTICAL],
        5: [ROTATE_270, FLIP_HORIZONTAL],
        6: [ROTATE_270],
        7: [ROTATE_90, FLIP_HORIZONTAL],
        8: [ROTATE_90],
    }

    def __init__(self, *args):
        """
        Possible arguments:
            - Transpose.AUTO
            - Transpose.FLIP_HORIZONTAL
            - Transpose.FLIP_VERTICAL
            - Transpose.ROTATE_90
            - Transpose.ROTATE_180
            - Transpose.ROTATE_270
        The order of the arguments dictates the order in which the
        Transposition steps are taken.
        If Transpose.AUTO is present, all other arguments are ignored, and
        the processor will attempt to rotate the image according to the
        EXIF Orientation data.
        """
        super(Transpose, self).__init__()
        if args:
            self.methods = args

    def rgba_to_rgb(self, img, mode='jpg'):
        fill_color = '#ffffff'
        background = Image.new('RGB', img.size, fill_color)

        split = img.split()
        if img.mode == 'P' and len(split) < 4:
            split = img.convert('RGBA').split()

        if len(split) == 4:
            background.paste(img, mask=split[3])
        else:
            background.paste(img)
        return background

    def process_binary(self, binary, format='jpeg', raise_on_open=False):
        assert type(binary) in (bytes, bytearray), f'Wrong binary type: {type(binary)}'
        try:
            try:
                img = Image.open(io.BytesIO(binary))
            except Exception:
                if raise_on_open:
                    raise TransposeError
                raise

            if self.AUTO in self.methods:
                try:
                    orientation = img._getexif()[0x0112]
                    ops = self._EXIF_ORIENTATION_STEPS[orientation]
                except (IndexError, KeyError, TypeError, AttributeError):
                    ops = []
            else:
                ops = self.methods
            for method in ops:
                self.log.debug('Operation: {}'.format(method))
                img = img.transpose(method)
            buf = io.BytesIO()
            if img.mode in ('RGBA', 'LA', 'P') and format.upper() in ('JPG', 'JPEG'):
                img = self.rgba_to_rgb(img)
            img.save(buf, format=format, quality=100)
            buf.seek(0)
            return buf.read()
        except TransposeError:
            raise
        except Exception:
            self.log.exception('During transpose binary:')
            return binary

    def process(self, orig_img, format=None):
        try:
            fname = orig_img.name
            if format is None:
                format = fname.split('.')[-1].upper()
                if format.startswith('JP'):
                    format = 'JPEG'
            buf = io.BytesIO(self.process_binary(orig_img.read(), format=format))
            # seek to the end of file
            buf.seek(0, 2)
            out = InMemoryUploadedFile(buf, "image", fname, orig_img.content_type, buf.tell(), None)
            return out
        except Exception:
            return orig_img



#!/usr/bin/env python3
"""
Internal helpers for backup tools
"""
import argparse
import asyncio
import logging
import os

from fan_tools.mon_server import MetricsServer


log = logging.getLogger('fan_tools.backup.utils')


def get_backup_server(args, BackupServer):
    enabled = True
    if not args.enabled:
        enabled = False
        backup_command = ''
    else:
        backup_command = os.environ.get('BACKUP_COMMAND')
        if not backup_command:
            BACKUP_DB_CONTAINER = os.environ.get('BACKUP_DB_CONTAINER')
            assert BACKUP_DB_CONTAINER, 'BACKUP_DB_CONTAINER variable must be specified'
            BACKUP_SCRIPT = os.environ.get('BACKUP_DB_SCRIPT', '/create_backup.py')
            backup_command = f'docker exec -i {BACKUP_DB_CONTAINER} {BACKUP_SCRIPT}'

    return BackupServer(
        prefix=args.prefix,
        metric_name=args.metric_name,
        bucket_name=args.bucket,
        backup_command=backup_command,
        enabled=enabled,
    )


def parse_args(environ_defaults={}):
    parser = argparse.ArgumentParser(description='DESCRIPTION')
    parser.add_argument('-m', '--monitoring-group', default='backups')
    parser.add_argument('-b', '--bucket')
    parser.add_argument('-d', '--daemonize', action='store_true', help='daemonize')
    parser.add_argument('-p', '--prefix', default=os.environ.get('BACKUP_PREFIX', 'backups/'))
    parser.add_argument(
        '--enabled',
        action='store_true',
        default=os.environ.get('ENABLE_BACKUP', 'false').lower() != 'false',
    )
    parser.add_argument('--metric-name', default='backup_metric')
    parser.add_argument(
        '--skip-upload', action='store_true', help='Works only in non-daemonized execution'
    )
    # parser.add_argument('-m', '--mode', default='auto', choices=['auto', 'manual'])
    # parser.add_argument('-l', '--ll', dest='ll', action='store_true', help='help')
    defaults = {}
    for k, v in environ_defaults.items():
        if isinstance(v, list):
            name, default = v
            defaults[k] = os.environ.get(name, default)
        else:
            defaults[k] = os.environ.get(v)
    parser.set_defaults(**defaults)
    return parser.parse_args()


def get_app(args, backup_server):
    from fastapi import FastAPI

    app = FastAPI(on_startup=[backup_server.run_monitoring])
    MetricsServer(app, args.monitoring_group)
    return app


def run_main(backup_server, environ_defaults={}, args=None):
    if not args:
        args = parse_args(environ_defaults)
    backup_server = get_backup_server(args, backup_server)
    if args.daemonize:
        import uvicorn

        app = get_app(args, backup_server)
        uvicorn.run(app, host='0.0.0.0', port=os.environ.get('MONITORING_PORT', 80))
    else:
        asyncio.run(backup_server.perform_backup(skip_upload=args.skip_upload))



import asyncio
import pathlib
import shutil
from typing import List, Union

from starlette.background import BackgroundTask
from starlette.requests import Request
from starlette.responses import JSONResponse, Response
from starlette.types import Receive, Scope, Send


class ValidationError(Exception):
    def __init__(self, message: str, status_code: int = 400):
        self.message = message
        self.status_code = status_code
        super().__init__(self.message)


async def remove_temp_folder(folder_path: pathlib.Path):
    try:
        shutil.rmtree(folder_path)
    except OSError:
        pass


def remove_temp_folder_task(folder_path: pathlib.Path):
    return BackgroundTask(remove_temp_folder, folder_path=folder_path)


def get_timeout(request_data: dict, param_name: str = "timeout", default: int = 60) -> int:
    timeout = default
    if timeout_data := request_data.get(param_name):
        try:
            timeout = int(timeout_data)
        except ValueError:
            pass
    return timeout


class BaseMetricsView:
    def __init__(self, *args):
        self.handlers = args

    async def get_default(self) -> List[str]:
        return []

    def get_response_handler(self):
        return Response

    async def get_handlers_data(self, request: Request) -> List[str]:
        data = []
        for handler in self.handlers:
            if asyncio.iscoroutinefunction(handler):
                res = await handler(request=request)
            else:
                res = handler(request=request)
            if isinstance(res, dict):
                res = [res]
            data.extend(res)
        return data

    async def get_response(self, request: Request) -> Response:
        data = await self.get_default()
        handlers_data = await self.get_handlers_data(request)
        resp_data = data + handlers_data
        return self.get_response_handler()('\n'.join(resp_data), media_type="text/plain")

    async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
        assert scope["type"] == "http"
        request = Request(scope, receive)
        response = await self.get_response(request=request)
        await response(scope, receive, send)


class HealthView(BaseMetricsView):
    async def get_default(self) -> dict:
        return {"status": "OK"}

    def get_response_handler(self):
        return JSONResponse

    async def get_response(self, request: Request) -> Response:
        data = await self.get_default()
        handlers_data = await self.get_handlers_data(request)
        data.update(*handlers_data)
        return self.get_response_handler()(data)


class PrometheusView(BaseMetricsView):
    async def get_default(self):
        return ["up 1"]


metric_storage = {}


class MetricStorage:
    def increment(self, metric_name: str) -> int:
        if metric_name not in metric_storage:
            metric_storage[metric_name] = 0
        if not isinstance(metric_storage[metric_name], int):
            raise ValueError('Metric not support incrementing')
        metric_storage[metric_name] += 1
        return metric_storage[metric_name]

    def get_metrics(self):
        return metric_storage

    def push(self, metric_name: str, value: Union[str, int, float]) -> Union[str, int, float]:
        if metric_name not in metric_storage:
            metric_storage[metric_name] = []
        if not isinstance(metric_storage[metric_name], list):
            raise ValueError('Metric not support pushing')
        metric_storage[metric_name].append(value)
        return metric_storage[metric_name]

    def reset(self):
        metric_storage.clear()



from functools import cached_property

from django.conf import settings
from opentelemetry.instrumentation.django.middleware import _DjangoMiddleware
from opentelemetry.trace import get_tracer, get_tracer_provider

from fan_tools.otel import instrument_logging, instrument_psycopg2
from fan_tools.otel.jaeger_tracing import setup_jaeger_tracer


class OtelDjangoMiddleware(_DjangoMiddleware):
    @cached_property
    def _tracer(self):
        tracer_provider = get_tracer_provider()
        tracer = get_tracer(
            'django',
            '0.1',
            tracer_provider=tracer_provider,
        )
        return tracer


def instrument_django():
    settings.MIDDLEWARE.insert(0, 'fan_tools.otel.OtelDjangoMiddleware')


INS_DJANGO = [instrument_django, instrument_logging, instrument_psycopg2]


def enable_otel(env, service, host='otlp', port=6831, instrumentations=INS_DJANGO, additional={}):
    for instrument in instrumentations:
        instrument()
    setup_jaeger_tracer(env, service, host, port, additional)



from google.cloud import storage

from fan_tools.backup import BackupMonitoring


class GCloud(BackupMonitoring):
    def __init__(self, *args, bucket_name=None, prefix='backup/', **kwargs):
        # from google.cloud import storage
        super().__init__(*args, **kwargs)
        assert bucket_name, 'You should define bucket-name'
        self.prefix = prefix
        self.cli = storage.Client()
        self.bucket = self.cli.get_bucket(bucket_name)

    def get_backups_list(self):
        items = list(self.bucket.list_blobs(prefix=self.prefix))
        return [s.name.replace(self.prefix, '') for s in items]

    def perform_upload(self, src_name, dst_name):
        self.log.debug('Upload {src_name} => {dst_name}')
        blob = self.bucket.blob(dst_name)
        blob.upload_from_filename(src_name)



#!/usr/bin/env python3

import argparse
import inspect
import json
import os
import sys
from functools import partial
from importlib import import_module

from fan_tools.doc_utils.fan_sphinx.dyn_json import serializer_doc_info

from django.core.serializers.json import DjangoJSONEncoder
from rest_framework_dyn_serializer import DynModelSerializer


def compose(*funs):
    it = funs[-1]
    funs = funs[:-1]

    def inner(*args, **kwargs):
        for res in it(*args, **kwargs):
            for fun in funs:
                res = fun(res)
            if inspect.isgenerator(res):
                for r in res:
                    yield r
            else:
                yield res

    return inner


def get_modules(apps):
    for app in apps:
        for mod in ('serializers', 'forms'):
            try:
                yield import_module('{}.{}'.format(app, mod))
            except ImportError:
                pass
            except AttributeError:
                pass


def get_dynserializers(module):
    filters = (
        lambda i: inspect.isclass(i),
        lambda i: i != DynModelSerializer,
        lambda i: not (hasattr(i, 'Meta') and getattr(i.Meta, 'abstract', False)),
        lambda i: issubclass(i, DynModelSerializer) or getattr(i, '_write_docs', False),
    )

    for item in dir(module):
        item = getattr(module, item)
        if all(f(item) for f in filters):
            yield item


class ArtifactWriter:
    _inst = None

    def __init__(self):
        path = os.environ.get('DOCS_ROOT', './.doc')
        if not os.path.exists(path):
            os.mkdir(path)
        self.doc_root = path

    @classmethod
    def instance(cls):
        if not cls._inst:
            cls._inst = cls()
        return cls._inst

    def dump(self, data, path):
        f_name = os.path.join(self.doc_root, '{}.json'.format(path))
        with open(f_name, 'w') as f:
            json.dump(data, f, cls=DjangoJSONEncoder)


def container_type():
    container = os.environ.get('CONTAINER_TYPE')
    if not container:
        sys.exit('Error: CONTAINER_TYPE is not defined')
    return container


def process_item(item, flags):
    path = '{}.{}'.format(item.__module__, item.__name__)
    if flags.rst:
        print('.. dyn_serializer:: {}/{}\n'.format(container_type(), path))
    if flags.artifacts:
        data = serializer_doc_info(item, path)
        ArtifactWriter.instance().dump(data, path)


def process(apps, fun):
    f = compose(get_dynserializers, get_modules)(apps)
    for s in f:
        fun(s)


parser = argparse.ArgumentParser(description='Parse serializers sources')

parser.add_argument('--rst', action='store_true', default=False, help='Output rst with serializers')
parser.add_argument(
    '--artifacts', action='store_true', default=False, help='Write serializers artifacts'
)


def main():
    sys.path.append('.')
    args = parser.parse_args()

    if not any((args.rst, args.artifacts)):
        parser.print_help()

    import django

    django.setup()
    from django.apps import apps

    all_apps = (app.module.__name__ for app in apps.get_app_configs())

    process(all_apps, partial(process_item, flags=args))


if __name__ == '__main__':
    main()



from contextlib import contextmanager
from typing import Any, Dict

from opentelemetry.propagate import extract, inject
from opentelemetry.trace import get_tracer, SpanKind


@contextmanager
def extract_span(
    headers: Dict[str, Any], mod_name: str, name: str, kind: SpanKind = SpanKind.CLIENT
):
    ctx = extract(headers)
    tracer = get_tracer(mod_name)
    with tracer.start_as_current_span(name, context=ctx, kind=kind) as span:
        yield span



#!/usr/bin/env python3

import argparse
from datetime import datetime, timedelta
from typing import Optional

import boto3
import pytz
from dateutil.parser import parse


LAST_VERSIONS_COUNT = 4
MINIMUM_PERIOD_DAYS = 14


def get_alias_from_name(name: str) -> Optional[str]:
    parts = name.split(':')

    if len(parts) == 1:
        return
    return parts[-1]


def main():
    arg_parser = argparse.ArgumentParser()
    arg_parser.add_argument('--last-versions-count', type=int, default=LAST_VERSIONS_COUNT)
    arg_parser.add_argument('--minimum-period-days', type=int, default=MINIMUM_PERIOD_DAYS)
    arg_parser.add_argument('--dry', action='store_true')
    arg_parser.add_argument('--aws-access-key-id', required=False)
    arg_parser.add_argument('--aws-secret-access-key', required=False)
    arg_parser.add_argument('--region-name', required=False)
    arg_parser.add_argument('--function-name', required=False)
    arg_parser.add_argument('--actual-names', nargs='+', required=False, default=set())
    arg_parser.add_argument('--ecr-repo-name')

    args = arg_parser.parse_args()

    aws_kwargs = {}

    if args.aws_access_key_id is not None:
        aws_kwargs['aws_access_key_id'] = args.aws_access_key_id
    if args.aws_secret_access_key is not None:
        aws_kwargs['aws_secret_access_key'] = args.aws_secret_access_key
    if args.region_name is not None:
        aws_kwargs['region_name'] = args.region_name

    dry = args.dry

    if dry:
        print('Dry run!')

    lambda_client = boto3.client('lambda', **aws_kwargs)
    ecr_client = boto3.client('ecr', **aws_kwargs)

    actual_from = datetime.now(tz=pytz.utc) - timedelta(days=args.minimum_period_days)

    function_name = args.function_name
    actual_names = set(args.actual_names)
    repo_name = args.ecr_repo_name

    print(f'Clean for {function_name}')
    all_versions = lambda_client.list_versions_by_function(FunctionName=function_name)['Versions']
    all_versions = list(filter(lambda v: v['Version'] != '$LATEST', all_versions))
    all_versions.sort(key=lambda v: int(v['Version']), reverse=True)
    actual_aliases = set(filter(lambda k: k is not None, map(get_alias_from_name, actual_names)))
    available_aliases = {
        a['Name']: a
        for a in lambda_client.list_aliases(FunctionName=function_name)['Aliases']
    }
    actual_versions = {v['Version'] for v in all_versions[:args.last_versions_count]}
    actual_images_uri = set()

    for version_data in all_versions:
        if parse(version_data['LastModified']) >= actual_from:
            actual_versions.add(version_data['Version'])

    for alias in actual_aliases:
        actual_versions.add(available_aliases.pop(alias)['FunctionVersion'])

    for v in actual_versions:
        resp = lambda_client.get_function(FunctionName=function_name, Qualifier=v)
        if resp['Code'].get('ImageUri') is not None:
            actual_images_uri.add(resp['Code']['ImageUri'])

    unused_aliases = [
        a for a in available_aliases.values() if a['FunctionVersion'] not in actual_versions
    ]
    to_del_aliases = {a['Name'] for a in unused_aliases}
    to_del_versions = {v['Version'] for v in all_versions} - actual_versions

    for alias in to_del_aliases:
        print(f'Delete alias: {alias}')
        if not dry:
            lambda_client.delete_alias(FunctionName=function_name, Name=alias)

    to_delete_image_tags = set()

    for version in to_del_versions:
        print(f'Delete version: {version}')

        image_uri = lambda_client.get_function(
            FunctionName=function_name,
            Qualifier=version,
        )['Code'].get('ImageUri')

        if not dry:
            lambda_client.delete_function(FunctionName=function_name, Qualifier=version)

        if image_uri is not None and image_uri not in actual_images_uri:
            print(f'Delete image uri: {image_uri}')
            to_delete_image_tags.add(image_uri.split(':')[-1])

    if to_delete_image_tags:
        print(f'Delete tags {to_delete_image_tags}')
        if not dry:
            ecr_client.batch_delete_image(
                repositoryName=repo_name,
                imageIds=[{'imageTag': t} for t in to_delete_image_tags],
            )


if __name__ == '__main__':
    main()



from opentelemetry import trace
from opentelemetry.exporter.jaeger.thrift import JaegerExporter
from opentelemetry.sdk.resources import Resource
from opentelemetry.sdk.trace import TracerProvider
from opentelemetry.sdk.trace.export import SimpleSpanProcessor

from fan_tools.otel import instrument_logging


def setup_jaeger_tracer(env, service, host, port, additional={}):
    resource = Resource(
        attributes={'service.name': service, 'env': env, 'dd.service': service, **additional}
    )
    trace.set_tracer_provider(TracerProvider(resource=resource))
    trace.get_tracer_provider().get_tracer(__name__)

    jaeger_exporter = JaegerExporter(
        agent_host_name=host,
        agent_port=port,
    )

    trace.get_tracer_provider().add_span_processor(SimpleSpanProcessor(jaeger_exporter))


def enable_otel(
    env, service, host='otlp', port=6831, instrumentations=[instrument_logging], additional={}
):
    for instrument in instrumentations:
        instrument()
    setup_jaeger_tracer(env, service, host, port, additional)



#!/usr/bin/env python3
"""
Export items in prometheus format.
"""
import asyncio
import logging
import os
from collections import ChainMap
from functools import partial

from fastapi import APIRouter, FastAPI
from fastapi.responses import PlainTextResponse


log = logging.getLogger('fan_monitoring')
router = APIRouter()


class MetricsServer:
    """
    You you should add monitoring functions with add_task
    Your update function will receive update_metrics function with all additional args
    """

    def __init__(self, app: FastAPI, status_metric='running{example_var="default_env"}'):
        self.app = app
        self.app.get('/metrics', response_class=PlainTextResponse)(self.expose_metrics)
        self.metrics = f'{status_metric} 1'
        self.all_names = [{status_metric: 1}]
        self.tasks = []
        self.running_tasks = []

    def update_metrics(self, names, metrics):
        for name in names:
            if name not in metrics:
                names[name] = 0

        for name, value in metrics.items():
            names[name] = value

        mlist = [f'{name} {value}' for name, value in ChainMap(*self.all_names).items()]
        self.metrics = '\n'.join(mlist)
        return names

    def gen_update_func(self):
        local_names = {}
        self.all_names.append(local_names)
        return partial(self.update_metrics, local_names)

    def add_task(self, loop_func, *args, **kwargs):
        self.tasks.append({'func': loop_func, 'args': args, 'kwargs': kwargs})
        self.app.on_event('startup')(self.run_tasks)

    async def run_tasks(self):
        print(f'RUN TASKS: {self.tasks}')
        for task in self.tasks:
            loop_func = task['func']
            coro = loop_func(self.gen_update_func(), *task['args'], **task['kwargs'])
            self.running_tasks.append(asyncio.create_task(coro))

    async def finish_tasks(self):
        for task in self.runing_tasks:
            await task.cancel()

    async def expose_metrics(self):
        return self.metrics


def main():
    import uvicorn

    from fan_tools.mon_server.certs import update_certs_loop

    app = FastAPI()
    mserver = MetricsServer(app)
    mserver.add_task(update_certs_loop, hosts=['perfectlabel.io', 'robopickles.com'])
    uvicorn.run(app, host='0.0.0.0', port=os.environ.get('MONITORING_PORT', 8000))


if __name__ == '__main__':
    main()



"""
check usage in examples/send_dd_metric.py
"""

import logging
import os


metric_logger = logging.getLogger('fan.metric')
ENV = os.environ.get('ENV', 'not_set')
SERVICE = os.environ.get('SERVICE', 'not_set')
COMMON = {
    'env': ENV,
    'service': SERVICE,
}
if version := os.environ.get('GITHUB_SHA'):
    COMMON['version'] = version


def send_error_metric(name: str, tags={}, increment=1):
    """
    Shortcut to bypass all fuss with monitoring, just call when things must not happen
    Usually you want to setup some kind of notification for such metric
    """
    metric_logger.error(
        {
            'tags': {
                **COMMON,
                **tags,
                'error_type': name,
            },
            'increment': increment,
            'message': 'error_metric',
        }
    )


def send_metric(name: str, tags={}, increment=1):
    metric_logger.info(
        {
            'tags': {
                **COMMON,
                **tags,
            },
            'message': name,
            'increment': increment,
        }
    )



import logging.config
import os
import sys

from pythonjsonlogger.jsonlogger import JsonFormatter


class JSFormatter(JsonFormatter):
    msg_keys = [
        'asctime',
        'created',
        'filename',
        'funcName',
        'levelname',
        'levelno',
        'lineno',
        'module',
        'msecs',
        'message',
        'name',
        'pathname',
        'process',
        'processName',
        'relativeCreated',
        'thread',
        'threadName',
    ]

    def __init__(self, *args, env_vars=[], **kwargs):
        super(JSFormatter, self).__init__(*args, **kwargs)
        self.default_keys = {k: v for k, v in list(os.environ.items()) if k in env_vars}
        self._required_fields.extend(self.msg_keys)

    def process_log_record(self, rec):
        rec.update(self.default_keys)
        return rec


def base_handler(filename, **params):
    return {
        'level': 'DEBUG',
        'class': 'safe_logger.TimedRotatingFileHandlerSafe',
        'when': 'midnight',
        'backupCount': 7,
        'formatter': 'standard',
        'filename': filename,
        **params,
    }


def get_plain_logname(base_name, root_dir, enable_json):
    """
    we nest all plain logs to prevent double log shipping
    """
    if enable_json:
        nested_dir = os.path.join(root_dir, 'plain')
        if os.path.exists(root_dir) and not os.path.exists(nested_dir):
            os.mkdir(nested_dir)
        root_dir = nested_dir
    return os.path.join(root_dir, '{}.log'.format(base_name))


def setup_logger(
    base_name,
    root_dir=None,
    enable_json=True,
    json_formatter='fan_tools.fan_logging.JSFormatter',
    loggers={},
    enable_stdout=True,
    stdout_level='DEBUG',
    handlers={},
    json_params={}
):
    """
    json_formatter:
    'fan.contrib.django.span_formatter.SpanFormatter' - add INSTALLATION_ID, SPAN and etc
    """
    if not root_dir:
        root_dir = os.environ.get('LOG_DIR')
    assert root_dir, 'You should pass root_dir parameter or set env LOG_DIR'

    JSON_FORMATTER = {
        '()': json_formatter,
        'env_vars': ['HOST_TYPE', 'DEPLOYMENT_CONFIG', 'DEPLOYMENT_BRANCH', 'CONTAINER_TYPE'],
        **json_params,
    }

    default_loggers = {
        '': {'handlers': ['default'], 'level': 'DEBUG', 'propagate': True},
        'googleapicliet.discovery_cache': {'level': 'ERROR'},
        'boto3': {'level': 'INFO'},
        'botocore': {'level': 'INFO'},
        'kazoo': {'level': 'INFO'},
        'urllib3': {'level': 'INFO'},
    }

    LOGGING = {
        'version': 1,
        'disable_existing_loggers': False,
        'formatters': {
            'json': JSON_FORMATTER,
            'standard': {'format': '%(asctime)s [%(levelname)s] %(name)s: %(message)s'},
        },
        'handlers': {'default': base_handler(get_plain_logname(base_name, root_dir, enable_json))},
        'loggers': {**default_loggers, **loggers},
    }
    if enable_json:
        LOGGING['handlers']['json'] = base_handler(
            os.path.join(root_dir, '{}.json_log'.format(base_name)), formatter='json'
        )
        LOGGING['loggers']['']['handlers'].append('json')

    if handlers:
        LOGGING['handlers'].update(handlers)
        for handler_name in handlers:
            LOGGING['loggers']['']['handlers'].append(handler_name)

    if enable_stdout:
        LOGGING['handlers']['stdout'] = {
            'level': stdout_level,
            'class': 'logging.StreamHandler',
            'stream': sys.stdout,
            'formatter': 'standard',
        }
        LOGGING['loggers']['']['handlers'].append('stdout')
    logging.config.dictConfig(LOGGING)


def setup_fan_logger(base_name, root_dir=None):
    return setup_logger(
        base_name,
        root_dir,
        enable_json=True,
        json_formatter='fan.contrib.django.span_formatter.SpanFormatter',
    )



from rest_framework.pagination import PageNumberPagination
from rest_framework.pagination import _positive_int


class ApiPageNumberPagination(PageNumberPagination):
    ''' Allow turn off pagination by specifying zero page_zize.'''

    page_size_query_param = 'page_size'

    def get_page_size(self, request):
        """
        Disable pagination by 'listening' for a zero value for
        page_size_query_param
        """
        if self.page_size_query_param:
            try:
                return _positive_int(
                    request.query_params[self.page_size_query_param],
                    strict=False,
                    cutoff=self.max_page_size
                )
            except (KeyError, ValueError):
                pass

        return self.page_size



import logging


ENV_VARS = {
    'ENV',
    'HOST_TYPE',
    'DEPLOYMENT_CONFIG',
    'DEPLOYMENT_BRANCH',
    'CONTAINER_TYPE',
    'SERVICE',
}
LOG_LEVELS = {
    logging.WARN: ['websockets.protocol', 'websockets.server', 'uvicorn.error', 'pika', 'PIL'],
    logging.INFO: [
        's3transfer.tasks',
        's3transfer.utils',
        's3transfer.futures',
        'aioredis',
        'asyncio',
        'faker.factory',
        'factory.generate',
    ],
}

OTEL = {
    'VECTOR': ('vector', 9998),
}
